package descartes.kieker4dql.dqlconnector.structures;


import java.util.HashMap;

//import org.apache.commons.logging.Log;
//import org.apache.commons.logging.LogFactory;



public abstract class AbstractCollectorMap<K,V> extends HashMap<K,V> {

	/**
	 * 
	 */
	private static final long serialVersionUID = -9053095852039454705L;
	// private static final Log log =
	// LogFactory.getLog(AbstractCollectorMap.class);
	private long mapIDcounter=0;

	/**
	 * This Override ensures, that a certain key-value-pair is put into the map.
	 * If the map does not already contain the key, the map is updated with the new key-value-pair and the value will be retured.
	 * Otherwise the method "isUpdateValue" is invoked, which defines further update behavior to enhance any value object.
	 * Furthermore "null" is returned;
	 */
	@SuppressWarnings("unchecked")
	@Override
	public V put(K key,V newValue){

		Object dummy = super.get(key);
		if(dummy == null){
			mapIDcounter++;
			super.put(key, newValue);
			return newValue;
		} else {
			V combinedValue= null;
			Object combinedObject = "";
			try {
				combinedObject = isUpdateValue(key,newValue);
				if (combinedObject !=null) {
					combinedValue=(V) combinedObject;
					super.replace(key, newValue, combinedValue);
				}
			} catch (Exception e) {
				// log.error("casting error: object " + combinedObject + " is
				// not of type " + combinedValue.getClass(), new
				// IllegalArgumentException());
			}
			
		}
		return null;
	}

	public long getMapIDcounter() {
		return mapIDcounter;
	}
	
	
	/**
	 * When multiple values share the same key, this method is invoked by "put" to specify the behavior for this case.
	 * If this case cannot be avoided, it is recommended to use any type of List for values in general.
	 * @param key which uniquely identifies the original value
	 * @param newValue is the object that is required to be included
	 * @return combined value, which replaces the original value of the key
	 */
	protected abstract Object isUpdateValue(Object key, Object newValue);

	@Override
	public V get(Object key){
		V value = super.get(key);
		if(value != null){
			return value;
		}
		return null;
	}



	public synchronized String[] toArray(){
		String[] keyArray = new String[super.size()];
		int iterator=0;
		for (Entry<K, V> entry : super.entrySet()) {
			keyArray[iterator]=entry.getKey().toString();
			iterator++;
		}
		return keyArray;
	}

	@Override
	public synchronized String toString(){
		StringBuilder builder = new StringBuilder();

		for (Entry<K, V> entry : super.entrySet()) {
			builder.append("\n");
			builder.append(entry.getKey().toString());
			builder.append(" :: \n \t");
			builder.append(entry.getValue().toString());
			builder.append("\n");
		}
		builder.append("\n");
		return builder.toString();
	}
	
	public abstract void pushToTrace(Trace trace);

}
